//	Fish.cpp - Class representing a fish.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/TextLayout.h"
#include "drawing/Drawer.h"
#include "ColourHelpers.h"
#include "Random.h"
#include "Fish.h"

//------------------------------------------------------------------------------
Fish::Fish(const TwoFloats& pos,
		   const ThreeFloats& col,
		   const std::wstring& fishName,
		   int flowerIndex):
position(pos.x, -64.0f),
initialTween(position, pos, 0.015f),
positionTween(position, position, 0.05f),
fallTween(position, position, 0.01f),
colour(col),
name(fishName),
currentFlower(flowerIndex),
destFlower(flowerIndex),
noFall(false),
winner(false)
{
	initialTween.setDelay(Random::getGlobal().getRandomFloat() * 0.25f);
	initialTween.reset(pos);
	positionTween.setIndex(1.0f);
	fallTween.setIndex(1.0f);

	timer = 240.0f;// + (Random::getGlobal().getRandomFloat() * 180.0f);
}

//------------------------------------------------------------------------------
Fish::~Fish()
{

}

//------------------------------------------------------------------------------
void Fish::initialUpdate(float delta)
{
	if(!initialTween.getFinished())
		initialTween.tick(delta);
	if(!fallTween.getFinished())
		fallTween.tick(delta);
	if(!positionTween.getFinished())
		positionTween.tick(delta);
}

//------------------------------------------------------------------------------
void Fish::update(float delta)
{
	TwoFloats tempPos;

	if(!initialTween.getFinished())
		initialTween.tick(delta);
	if(!fallTween.getFinished())
		fallTween.tick(delta);
	if(!positionTween.getFinished())
		positionTween.tick(delta);
	else if(fallTween.getFinished())
	{
		destKey = L"";
		currentFlower = destFlower;
	}

	if(initialTween.getFinished() &&
	   !winner &&
	   fallTween.getFinished() &&
	   !noFall)
	{
		if(timer > 0.0f)
			timer -= delta;
		if((timer <= 0.0f) && (timer > -500.0f))
		{
			tempPos = position;
			tempPos.y = 1024.0f;
			fallTween.reset(tempPos);
			timer = -1000.0f;

			destKey = getRandomKey();
		}
	}
}

//------------------------------------------------------------------------------
void Fish::draw(shared_ptr<Drawer>& d)
{
	TwoFloats tempPos(position);

	tempPos.x -= 31.0f;
	tempPos.y -= 31.0f;
	d->drawImage(L"Fish", tempPos, 1.0f, colour);

	tempPos.x += 31.0f;
	tempPos.x -= 128.0f;
	if(destKey != L"")
	{
		d->drawText(destKey,
					L"Font1",
					tempPos,
					256.0f,
					TextLayout::Centre);
	}

	if(!winner)
	{
		tempPos.y += 64.0f;
		d->drawText(name,
					L"Font0",
					textLayout,
					tempPos,
					256.0f,
					TextLayout::Centre);
	}
}

//------------------------------------------------------------------------------
void Fish::jumpTo(const TwoFloats& pos, int flowerIndex)
{
	fallTween.setIndex(1.0f);
	positionTween.reset(pos);

	timer = 60.0f + (Random::getGlobal().getRandomFloat() * 180.0f);

	destFlower = flowerIndex;
}

//------------------------------------------------------------------------------
void Fish::fall(bool includeKey)
{
	TwoFloats tempPos;

	tempPos = position;
	tempPos.y = 1024.0f;
	fallTween.reset(tempPos);

	destFlower = -1;

	if(includeKey)
		destKey = getRandomKey();
	else
		destKey = L"";
}

//------------------------------------------------------------------------------
void Fish::updateDestKey()
{
	destKey = getRandomKey();
}

//------------------------------------------------------------------------------
const std::wstring Fish::getRandomKey() const
{
	int index;
	std::vector<std::wstring> keys;

	keys.push_back(L"a");
	keys.push_back(L"b");
	keys.push_back(L"c");
	keys.push_back(L"d");
	keys.push_back(L"e");
	keys.push_back(L"f");
	keys.push_back(L"g");
	keys.push_back(L"h");
	keys.push_back(L"i");
	keys.push_back(L"j");
	keys.push_back(L"k");
	keys.push_back(L"l");
	keys.push_back(L"m");
	keys.push_back(L"n");
	keys.push_back(L"o");
	keys.push_back(L"p");
	keys.push_back(L"q");
	keys.push_back(L"r");
	keys.push_back(L"s");
	keys.push_back(L"t");
	keys.push_back(L"u");
	keys.push_back(L"v");
	keys.push_back(L"w");
	keys.push_back(L"x");
	keys.push_back(L"y");
	keys.push_back(L"z");

	index = Random::getGlobal().getRandomInt(keys.size());

	return keys[index];
}

//------------------------------------------------------------------------------
void Fish::setNoFall(bool val)
{
	noFall = val;
}

//------------------------------------------------------------------------------
void Fish::setWinner()
{
	winner = true;
	destKey = L"";
	fallTween.setIndex(1.0f);
	positionTween.reset(TwoFloats(640.0f, 300.0f));
}
