//	Fish.h - Class representing a fish.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FISH_H_
#define FISH_H_

#include "drawing/TextLayout.h"
#include "c0xHeaders.h"
#include "TwoFloats.h"
#include "Tweener.h"

#include <string>

class Drawer;

///	Class representing a fish.
class Fish
{
  public:
	///	Constructor.
	Fish(const TwoFloats& pos,
		 const ThreeFloats& col,
		 const std::wstring& fishName,
		 int flowerIndex);
	///	Destructor.
	~Fish();

	///	Bounces the fish down from the top of the screen.
	void initialUpdate(float delta);
	///	Updates the fish's timer.
	void update(float delta);
	///	Draws the fish.
	void draw(shared_ptr<Drawer>& d);

	///	Sets the next position to jump the fish to.
	void jumpTo(const TwoFloats& pos, int flowerIndex);
	///	Tells the fish to fall off the screen.
	void fall(bool includeKey = true);

	///	Updates the fish's destination key.
	void updateDestKey();

	///	Returns the fish's position.
	const TwoFloats& getPosition() const {return position;};
	///	Returns the fish's colour.
	const ThreeFloats& getColour() const {return colour;};
	///	Returns the fish's name.
	const std::wstring& getName() const {return name;};

	///	The fish's current destination key.
	const std::wstring& getDestKey() const {return destKey;};
	///	Returns true if the fish is at a flower and not falling.
	bool getFishIsStatic() const {return (destKey != L"") && positionTween.getFinished() && fallTween.getFinished();};
	///	Returns true if the fish is falling.
	bool getFishIsFalling() const {return !(fallTween.getFinished());};

	///	Returns the fish's destination flower.
	int getDestFlower() const {return destFlower;};

	///	Tells the fish not to fall off its flower.
	void setNoFall(bool val);

	///	Tells the fish it's the winner.
	void setWinner();
  private:
	///	Helper method. Returns a random destination key.
	const std::wstring getRandomKey() const;

	///	The fish's position.
	TwoFloats position;
	///	Used to bounce the fish down from the top of the screen.
	BounceTweener<TwoFloats> initialTween;
	///	Used to jump the fish between flowers.
	STweener<TwoFloats> positionTween;
	///	Used to fall the fish off the screen.
	ExpTweener<TwoFloats> fallTween;
	///	The fish's colour.
	ThreeFloats colour;
	///	The fish's name.
	std::wstring name;
	///	The fish's timer.
	float timer;

	///	The fish's text layout.
	TextLayout textLayout;

	///	The index of the fish's current flower.
	int currentFlower;
	///	The index of the fish's destination flower.
	int destFlower;

	///	The fish's current destination key.
	std::wstring destKey;

	///	True if the fish is to stay on the current flower with no fall count.
	bool noFall;

	///	True if this fish is the winner.
	bool winner;
};

#endif
